const gulp = require('gulp'),
  sass = require('gulp-sass'),
  minifycss = require('gulp-minify-css'),
  uglify = require('gulp-uglify'),
  rename = require('gulp-rename'),
  concat = require('gulp-concat'),
  gzip = require('gulp-gzip'),
  del = require('del'),
  nunjucks = require('gulp-nunjucks'),
  babel = require('gulp-babel');

const scriptsDeps = () =>
  gulp.src([
    'node_modules/jquery/dist/jquery.min.js',
    'node_modules/jquery-ui-dist/jquery-ui.min.js',
    'node_modules/moment/moment.js',
    'node_modules/moment-range/dist/moment-range.js',
    'node_modules/universal-geocoder/dist/universal-geocoder.min.js'
  ])
  .pipe(concat('deps.js'))
  .pipe(gulp.dest('web/build'));

const scriptsLibs = () =>
  gulp.src([
    'src/js/libs/**/!(leaflet-routing-machine)*.js',
    'src/js/libs/leaflet-routing-machine.js',
    'src/js/libs/bootstrap-datepicker.js' ,
    'src/js/libs/datepicker-locales/**/*.js' ,
    '!src/js/libs/materialize/unused/**/*.js'
  ])
  .pipe(concat('libs.js'))
  .pipe(gulp.dest('web/build'));

const templates = () =>
  gulp.src(['src/views/**/*.html.njk', '!src/views/nearest-mode/**/*.html.njk'])
    .pipe(nunjucks.precompile())
    .pipe(concat('templates.js'))
    .pipe(gulp.dest('web/build'));

const styles = () =>
  gulp.src(['src/scss/**/*.scss'])
    .pipe(sass({ includePaths: ['node_modules']}).on('error', sass.logError))
    .pipe(gulp.dest('web/assets'));

const prodStyles = () =>
  gulp.src('dist/*!(*.min).css')
    .pipe(rename({ suffix: '.min' }))
    .pipe(minifycss())
    .pipe(gulp.dest('dist'));

const gzipStyles = () =>
  gulp.src('dist/*.min.css')
    .pipe(gzip())
    .pipe(gulp.dest('dist'));

const concatDirectory = () =>
  gulp.src([
    'web/build/deps.js',
    'web/build/libs.js',
    'web/build/templates.js',
    'web/directory.js'
  ])
    .pipe(concat('gogocarto.js'))
    .pipe(gulp.dest('dist'));

const concatCss = () =>
  gulp.src(['web/assets/gogocarto.css',
    'web/assets/images/styles.css'
  ])
    .pipe(concat('gogocarto.css'))
    .pipe(gulp.dest('dist'));

const fontAssets = () =>
  gulp.src(['web/assets/images/fonts/*'])
    .pipe(gulp.dest('dist/fonts/'));

const imageAssets = () =>
  gulp.src(['web/assets/images/*.png'])
    .pipe(gulp.dest('dist/images/'));

const prodJs = () =>
  gulp.src(['dist/*!(*.min).js'])
    .pipe(rename({ suffix: '.min' }))
    .pipe(babel({
      presets: ['@babel/env']
    }))
    .pipe(uglify())
    .pipe(gulp.dest('dist'));

const gzipJs = () =>
  gulp.src(['dist/*.min.js'])
    .pipe(gzip())
    .pipe(gulp.dest('dist'));

const cleanBuild = () => del(['web/build']);

const cleanDist = () => del(['dist']);

const watchStyles = () => gulp.watch(['src/scss/**/*.scss'], styles);
const watchDeps = () => gulp.watch('src/js/node_modules/**/*.js', scriptsDeps);
const watchLibs = () => gulp.watch('src/js/libs/**/*.js', scriptsLibs);
const watchTemplates = () => gulp.watch('src/views/**/*.njk', templates);

exports.watch = gulp.parallel(watchStyles, watchDeps, watchLibs, watchTemplates);
exports.build = gulp.series(cleanBuild, gulp.parallel(styles, scriptsDeps, scriptsLibs, templates));
exports.cleanDist = cleanDist;
exports.dist = gulp.series(cleanDist, gulp.parallel(
                  gulp.series(concatDirectory, prodJs, gzipJs),
                  gulp.series(concatCss, prodStyles, gzipStyles),
                  fontAssets, imageAssets));